﻿// // ******************************************************************
// //       /\ /|       @File         UsbPacket.h
// //       \ V/        @Brief
// //       | "")       @Author        lijinwen, ghz005@uni-trend.com.cn
// //       /  |        @Creation      2024-3-14
// //      /  \\        @Modified      2024-5-13
// //    *(__\_\
// // ******************************************************************

#pragma once
#include <typeinfo>
#include <vector>

#include "../BaseHelper/CommonHelper.h"
 
#include "../EdgePulse.h"

#include "SYNC.h"
#include "ProtocolUsbEnums.h"

namespace Protocol::ProtocolUsb
{
    class UsbPacket
    {
    public:
        UsbPacket();
        UsbPacket(TwoLevelEdgePulse* pidNode, int32_t& leftOverSize, SYNC sync, bool polarity);

        UsbPacketType Type() const { return type; }
        uint8_t PID() const { return pid; }
        void SetPID(uint8_t value) { pid = value; }
        bool LastNRBZDataBit() const;
        uint8_t RealPID() const { return static_cast<uint8_t>(pid & 0xF); }
        //const std::type_info& PIDType() const { return pidType; }
        //int PIDEndIndex() const { return pidEndIndex; }
        bool Polarity() const { return polarity; }
        const SYNC PacketSYNC() const { return packetSYNC; }
        const std::vector<uint8_t>& Address() const { return address; }
        uint8_t EndPoint() const { return endPoint; }
        const std::vector<uint8_t>& Datas() const { return datas; }
        uint8_t CRC5() const { return crc5; }
        bool IsValid() const { return isValid; }
        bool IsCRCChecked() const { return isCRCChecked; }

    private:
        UsbPacketType type = UsbPacketType::NoDefine;
        uint8_t pid;
        bool lastNRBZDataBit;
        //std::type_info& pidType;
        //int pidEndIndex;
        bool polarity;
        SYNC packetSYNC;
        std::vector<uint8_t> address;
        uint8_t endPoint;
        std::vector<uint8_t> datas;
        uint8_t crc5;
        bool isValid;
        bool isCRCChecked;

        static bool CheckNibblesInverse(uint8_t value);

        bool GetPID(TwoLevelEdgePulse* pidNode, int32_t& leftOverSize);

        bool DecodeNextByte(TwoLevelEdgePulse* node, int32_t& leftOverSize, uint8_t& outData);

        void DecodeFields(TwoLevelEdgePulse* node, int32_t& leftOverSize, const SYNC& sync);

        void DecodeFieldsByToken(TwoLevelEdgePulse* node, int32_t& leftOverSize, const SYNC& sync);
        void DecodeFieldsByData(TwoLevelEdgePulse* node, int32_t& leftOverSize, SYNC sync);

        void DecodeFieldsByHandShake(TwoLevelEdgePulse* node, int32_t& leftOverSize, SYNC sync) const;

        void DecodeFieldsBySpecial(TwoLevelEdgePulse* node, int32_t& leftOverSize, SYNC sync) const;
    };
}
